package com.vendor.burn.packagers.nand.ip;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.config.PreferenceConfig;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.define.IFastbootProgressMonitor;
import com.vendor.burn.packagers.nand.FlashOption;
import com.vendor.burn.packagers.nand.NandCRandom;
import com.vendor.burn.packagers.nand.PageType;
import com.vendor.burn.packagers.nand.ReversePageEccSector;
import com.vendor.burn.packagers.nand.ecc.ECC610;
import com.vendor.burn.packagers.nand.ecc.ECCType;
import com.vendor.burn.packagers.nand.ip.NandIPCommon;
import com.vendor.burn.preferences.FastbootPreferences;
import com.vendor.burn.utilitily.BytesUtils;
import com.vendor.burn.utilitily.FileUtils;
import com.vendor.burn.utilitily.ToolUtils;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.io.FileUtils;
import org.eclipse.core.runtime.IProgressMonitor;

public class NandIP610 extends NandIPCommon {
  private static Map<String, Integer> map = new HashMap<>();
  
  private static int NANDIP_610 = 1552;
  
  private static final int REVERSE_DATA_LENTH = 32;
  
  private static final ReversePageEccSector[] nfc610_page_ecc_reverse_sector = new ReversePageEccSector[] { 
      new ReversePageEccSector(NANDIP_610, PageType.Size8K, ECCType.Ecc24bit, 0), new ReversePageEccSector(NANDIP_610, PageType.Size4K, ECCType.Ecc24bit, 1), new ReversePageEccSector(NANDIP_610, PageType.Size8K, ECCType.Ecc41bit, 2), new ReversePageEccSector(NANDIP_610, PageType.Size16K, ECCType.Ecc41bit, 3), new ReversePageEccSector(NANDIP_610, PageType.Size2K, ECCType.Ecc4bit, 4), new ReversePageEccSector(NANDIP_610, PageType.Size4K, ECCType.Ecc4bit, 5), new ReversePageEccSector(NANDIP_610, PageType.Size2K, ECCType.Ecc24bit, 6), new ReversePageEccSector(NANDIP_610, PageType.Size16K, ECCType.Ecc60bit, 7), new ReversePageEccSector(NANDIP_610, PageType.Size8K, ECCType.Ecc60bit, 8), new ReversePageEccSector(NANDIP_610, PageType.Size16K, ECCType.Ecc80bit, 9), 
      new ReversePageEccSector(NANDIP_610, PageType.Size32K, ECCType.Ecc41bit, 10), new ReversePageEccSector(NANDIP_610, PageType.Size32K, ECCType.Ecc60bit, 11), new ReversePageEccSector(NANDIP_610, PageType.Size8K, ECCType.Ecc80bit, 12), new ReversePageEccSector(NANDIP_610, PageType.Size32K, ECCType.Ecc80bit, 13) };
  
  public NandIP610() {
    map.put("2k4bit", Integer.valueOf(60));
    map.put("2k13bit", Integer.valueOf(64));
    map.put("2k24bit", Integer.valueOf(116));
    map.put("2k27bit", Integer.valueOf(128));
    map.put("4k4bit", Integer.valueOf(88));
    map.put("4k13bit", Integer.valueOf(112));
    map.put("4k24bit", Integer.valueOf(200));
    map.put("4k27bit", Integer.valueOf(224));
    map.put("8k24bit", Integer.valueOf(368));
    map.put("8k27bit", Integer.valueOf(416));
    map.put("8k41bit", Integer.valueOf(608));
    map.put("8k60bit", Integer.valueOf(896));
    map.put("8k80bit", Integer.valueOf(1152));
    map.put("16k41bit", Integer.valueOf(1184));
    map.put("16k60bit", Integer.valueOf(1760));
    map.put("16k80bit", Integer.valueOf(1376));
  }
  
  public FlashOption[] getAvailableOptions(IBurner burner) {
    return new FlashOption[] { 
        new FlashOption(ECCType.Ecc4bit, PageType.Size2K, -1), new FlashOption(ECCType.Ecc13bit, PageType.Size2K, -1), new FlashOption(ECCType.Ecc24bit, PageType.Size2K, -1), new FlashOption(ECCType.Ecc27bit, PageType.Size2K, -1), new FlashOption(ECCType.Ecc4bit, PageType.Size4K, -1), new FlashOption(ECCType.Ecc13bit, PageType.Size4K, -1), new FlashOption(ECCType.Ecc24bit, PageType.Size4K, -1), new FlashOption(ECCType.Ecc27bit, PageType.Size4K, -1), new FlashOption(ECCType.Ecc24bit, PageType.Size8K, -1), new FlashOption(ECCType.Ecc27bit, PageType.Size8K, -1), 
        new FlashOption(ECCType.Ecc41bit, PageType.Size8K, -1), new FlashOption(ECCType.Ecc60bit, PageType.Size8K, -1), new FlashOption(ECCType.Ecc80bit, PageType.Size8K, -1), new FlashOption(ECCType.Ecc41bit, PageType.Size16K, -1), new FlashOption(ECCType.Ecc60bit, PageType.Size16K, -1), new FlashOption(ECCType.Ecc80bit, PageType.Size16K, -1) };
  }
  
  public byte[] genPageData(byte[] pagebuf, FlashOption option, int oobSize) {
    ECC610 ecc = new ECC610();
    if (option.getEccType() == ECCType.Ecc4bit) {
      if (option.getPageType() == PageType.Size2K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 14); 
      if (option.getPageType() == PageType.Size4K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 14); 
    } 
    if (option.getEccType() == ECCType.Ecc13bit) {
      if (option.getPageType() == PageType.Size2K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 14, 24); 
      if (option.getPageType() == PageType.Size4K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 14, 24); 
    } 
    if (option.getEccType() == ECCType.Ecc24bit) {
      if (option.getPageType() == PageType.Size2K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 42); 
      if (option.getPageType() == PageType.Size4K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 42); 
      if (option.getPageType() == PageType.Size8K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 42); 
    } 
    if (option.getEccType() == ECCType.Ecc27bit) {
      if (option.getPageType() == PageType.Size2K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 48); 
      if (option.getPageType() == PageType.Size4K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 48); 
      if (option.getPageType() == PageType.Size8K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 48); 
    } 
    if (option.getEccType() == ECCType.Ecc41bit) {
      if (option.getPageType() == PageType.Size8K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 72); 
      if (option.getPageType() == PageType.Size16K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 72); 
      if (option.getPageType() == PageType.Size32K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 62, 72); 
    } 
    if (option.getEccType() == ECCType.Ecc60bit) {
      if (option.getPageType() == PageType.Size8K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 108); 
      if (option.getPageType() == PageType.Size16K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 108); 
      if (option.getPageType() == PageType.Size32K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 62, 108); 
    } 
    if (option.getEccType() == ECCType.Ecc80bit) {
      if (option.getPageType() == PageType.Size8K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 140); 
      if (option.getPageType() == PageType.Size16K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 30, 140); 
      if (option.getPageType() == PageType.Size32K)
        return genPageDataBytesFor610(ecc, option, pagebuf, 62, 140); 
    } 
    return null;
  }
  
  public void make(String outputPath, FlashOption option, List<PartitionItem> partitions, int blockSize, boolean isSelectRandom, IFastbootProgressMonitor fastbootMonitor, IProgressMonitor monitor) {
    int usedOobSize = 0;
    usedOobSize = getUsedOobSize(option);
    fastbootMonitor.infoln("\nStart to make nand burner image...");
    fastbootMonitor.infoln("Options: " + option.toString() + "\n");
    monitor.beginTask(Messages.NandImageDialog_Title, partitions.size());
    int randomPageNum = -1;
    if (isSelectRandom && 
      blockSize > -1)
      if (option.getPageType().getPageSize() != PageType.Size8K
        .getPageSize()) {
        randomPageNum = blockSize / option.getPageType().getPageSize();
      } else {
        randomPageNum = blockSize / 8192;
      }  
    label123: for (PartitionItem item : partitions) {
      if (monitor.isCanceled())
        return; 
      boolean itemState = (!item.isChecked() || item.getFile() == null);
      if (itemState) {
        monitor.worked(1);
        continue;
      } 
      differenceBlockSize(item, blockSize);
      fastbootMonitor.info("Make image for '" + item.getFile().getName() + "'...");
      int pageCheckSize = option.getPageType().getPageSize();
      if (item.getFileSystem() == FileSystem.yaffs)
        pageCheckSize = option.getPageType().getPageSize() + 32; 
      File file = (item.getNewTempFile() == null) ? item.getFile() : item.getNewTempFile();
      int pageCount = (int)((file.length() + pageCheckSize - 1L) / pageCheckSize);
      if (item.getFileSystem() == FileSystem.yaffs)
        pageCount = (int)(file.length() / pageCheckSize); 
      File outFile = FileUtils.getFile(new String[] { outputPath + outputPath + File.separator + ".burn" });
      if (outFile.exists())
        FileUtils.deleteFile(outFile); 
      RandomAccessFile inRanFile = null;
      RandomAccessFile outRanFile = null;
      boolean ret = true;
      try {
        FileUtils.checkFilePath(file);
        inRanFile = new RandomAccessFile(file, "r");
        outRanFile = new RandomAccessFile(outFile, "rw");
        int pageMakeSize = option.getPageType().getPageSize() + 32;
        long start = item.getStart();
        int reversePageEccEec = 0;
        boolean page0ReverseFlag = false;
        boolean nandPinSaveMpde = false;
        if (ToolUtils.isCmdLine()) {
          nandPinSaveMpde = PreferenceConfig.isImgOpenPinSaveMode();
        } else {
          nandPinSaveMpde = FastbootPreferences.isNandPinSaveMpde();
        } 
        if (nandPinSaveMpde && 
          start == 0L) {
          PageType pageType = option.getPageType();
          ECCType eccType = option.getEccType();
          ReversePageEccSector pageSizeEccReverseSector = getPageSizeEccReverseSector(pageType, eccType);
          if (pageSizeEccReverseSector != null)
            reversePageEccEec = pageSizeEccReverseSector.getReverse_sec(); 
        } 
        for (int i = 0; i < pageCount; i++) {
          if (monitor.isCanceled()) {
            try {
              inRanFile.close();
            } catch (IOException iOException) {}
            try {
              outRanFile.close();
            } catch (IOException iOException) {}
            closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
            FileUtils.deleteFile(item.getNewTempFile());
            if (!ret) {
              FileUtils.deleteFile(outFile);
              monitor.worked(1);
              continue label123;
            } 
            return;
          } 
          byte[] pagebuf = BytesUtils.genBytesWithFF(pageMakeSize);
          try {
            inRanFile.seek((i * pageCheckSize));
            int read = inRanFile.read(pagebuf, 0, pageCheckSize);
            if (read < 0)
              System.out.println("Read Error."); 
            pagebuf[pageMakeSize - 1] = 0;
            pagebuf[pageMakeSize - 2] = 0;
          } catch (IOException e) {
            fastbootMonitor.warnln("FAILED! (Read data error.)");
            ret = false;
            break;
          } 
          byte[] data = null;
          try {
            data = genPageData(pagebuf, option, option.getOOBSize());
            if (data == null)
              break; 
            if (randomPageNum > -1) {
              NandCRandom random = new NandCRandom();
              if (usedOobSize != 0) {
                random.page_random_gen610(data, option.getPageType().getPageSize(), usedOobSize, i % randomPageNum);
              } else {
                random.page_random_gen610(data, option.getPageType().getPageSize(), option.getOOBSize(), i % randomPageNum);
              } 
            } 
            if (nandPinSaveMpde)
              if (start == 0L) {
                if (!page0ReverseFlag) {
                  int page0ReverseOff = 32 * reversePageEccEec;
                  reverseBuf(data, page0ReverseOff, 32);
                } 
                page0ReverseFlag = true;
              }  
            outRanFile.seek((i * (option.getPageType().getPageSize() + option.getOOBSize())));
            outRanFile.write(data);
            if (outRanFile.length() % (option.getPageType().getPageSize() + option.getOOBSize()) != 0L) {
              fastbootMonitor.warnln("FAILED! (Could not write output file.)");
              ret = false;
              break;
            } 
          } catch (Exception e) {
            fastbootMonitor.warnln("FAILED! (Data convert error.)");
            ret = false;
            break;
          } 
        } 
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
          continue;
        } 
      } catch (FileNotFoundException e) {
        fastbootMonitor.warnln("FAILED! (Source file not found.)");
        monitor.worked(1);
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
        } 
        continue;
      } catch (IOException iOException) {
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
          continue;
        } 
      } finally {
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
          continue;
        } 
      } 
      fastbootMonitor.info("OK!");
      fastbootMonitor.warnln("(Start Address = 0x" + 
          Long.toHexString(item.getStart()) + ", Length = 0x" + 
          Long.toHexString(item.getLength()) + ")");
      monitor.worked(1);
    } 
    fastbootMonitor
      .infoln("\nImage creating operation ends. All of the selected partitions are processed.");
    monitor.done();
  }
  
  private ReversePageEccSector getPageSizeEccReverseSector(PageType pagetype, ECCType ecctype) {
    ReversePageEccSector[] sector = nfc610_page_ecc_reverse_sector;
    for (int i = 0; i < sector.length; i++) {
      if (sector[i].getEcctype() == ecctype && sector[i]
        .getPagetype() == pagetype)
        return sector[i]; 
    } 
    return null;
  }
  
  private int getUsedOobSize(FlashOption option) {
    int usedOobSize = 0;
    if (option.getEccType() == ECCType.Ecc4bit)
      if (option.getPageType() == PageType.Size2K) {
        if (map.get("2k4bit") != null)
          return ((Integer)map.get("2k4bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size4K && 
        map.get("4k4bit") != null) {
        return ((Integer)map.get("4k4bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc13bit)
      if (option.getPageType() == PageType.Size2K) {
        if (map.get("2k13bit") != null)
          return ((Integer)map.get("2k13bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size4K && 
        map.get("4k13bit") != null) {
        return ((Integer)map.get("4k13bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc24bit)
      if (option.getPageType() == PageType.Size2K) {
        if (map.get("2k24bit") != null)
          return ((Integer)map.get("2k24bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size4K) {
        if (map.get("4k24bit") != null)
          return ((Integer)map.get("4k24bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size8K && 
        map.get("8k24bit") != null) {
        return ((Integer)map.get("8k24bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc27bit)
      if (option.getPageType() == PageType.Size2K) {
        if (map.get("2k27bit") != null)
          return ((Integer)map.get("2k27bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size4K) {
        if (map.get("4k27bit") != null)
          return ((Integer)map.get("4k27bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size8K && 
        map.get("8k27bit") != null) {
        return ((Integer)map.get("8k27bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc41bit)
      if (option.getPageType() == PageType.Size8K) {
        if (map.get("8k41bit") != null)
          return ((Integer)map.get("8k41bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size16K && 
        map.get("16k41bit") != null) {
        return ((Integer)map.get("16k41bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc60bit)
      if (option.getPageType() == PageType.Size8K) {
        if (map.get("8k60bit") != null)
          return ((Integer)map.get("8k60bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size16K && 
        map.get("16k60bit") != null) {
        return ((Integer)map.get("16k60bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc80bit)
      if (option.getPageType() == PageType.Size8K) {
        if (map.get("8k80bit") != null)
          return ((Integer)map.get("8k80bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size16K && 
        map.get("16k80bit") != null) {
        return ((Integer)map.get("16k80bit")).intValue();
      }  
    return usedOobSize;
  }
  
  private static void reverseBuf(byte[] buf, int startOff, int reverseLen) {
    for (int i = startOff; i < startOff + reverseLen; i++)
      buf[i] = (byte)(buf[i] ^ 0xFFFFFFFF); 
  }
  
  public String toString() {
    return "Nand IP 610";
  }
}
